from django.contrib.auth import login, logout
from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from django.http import HttpResponse
from django.shortcuts import redirect
from django.urls import reverse
from django.conf import settings

from urllib3 import PoolManager
from json import loads

import logging
from django_navbar_client.models import AuthProfile


PM = PoolManager()

logger = logging.getLogger(__name__)


def oauth_logout(request, **kwargs):
    user = request.user
    if user.is_anonymous:
        logger.warning("Anonymous logout attempted")
        return redirect(kwargs.get("next", reverse('home')))

    oauth_profile, created = AuthProfile.objects.get_or_create(user=user)
    if created:
        logger.warning("Profile Created during logout: %s", oauth_profile.user.username)
        return redirect(kwargs.get("next", reverse('home')))

    url = settings.OAUTH_SERVER_URL + "o/revoke_token/"
    headers = {"Content-Type": "application/x-www-form-urlencoded"}
    body = "token={}&client_id={}&client_secret={}".format(
        oauth_profile.token,
        settings.OAUTH_CLIENT_ID,
        settings.OAUTH_CLIENT_SECRET
    )

    r = PM.request(method='POST', url=url, headers=headers, body=body)
    if r.status == 200:
        logger.info("OAUTH token revoked server")
    else:
        logger.warning("OAuth Logout failed: %i", r.status)
        logger.warning("\tURL: %s", url)
        logger.warning("\tHEADERS: %s", headers)
        logger.warning("\tBODY: %s", body)
        logger.debug(r.data)

    url = settings.OAUTH_SERVER_URL + "api/logout/"
    headers = {"Authorization": oauth_profile.token}
    r = PM.request(method='GET', url=url, headers=headers)
    if r.status == 200:
        logger.info("Logged out in OAUTH server")
    else:
        logger.warning("OAuth Logout failed: %i", r.status)
        logger.warning("\tURL: %s", url)
        logger.warning("\tHEADERS: %s", headers)
        logger.debug(r.data)
    logout(request)
    logger.info("Logged out in local server")
    return redirect(kwargs.get("next", reverse('home')))


def oauth_login(request):
    logger.info("Redirect to login")
    login_url = "{0}o/authorize/?client_id={1}&response_type=code&redirect_uri=http://{2}{3}".format(
            settings.OAUTH_SERVER_URL,
            settings.OAUTH_CLIENT_ID,
            request.META['HTTP_HOST'], reverse("django_navbar_client:callback"))
    logging.debug("Redirect Oauth to %s", login_url)
    return redirect(login_url)


def oauth_navbar(request):
    caller = request.GET.get("caller")
    url = "{0}view/navbar/?caller={1}".format(
        settings.OAUTH_SERVER_URL,
        caller)
    logger.info("Ask navbar for %s at %s", request.user, request.session.session_key)
    if request.user.is_authenticated:
        auth_profile = request.user.authprofile

        remote_response = ask_oauth(url, auth_profile.token)
    else:
        remote_response = ask_oauth(url, None)
    if remote_response.status != 200:
        logger.error("Auth server returned an error(%s):\n  %s\n Request:\n  %s  ",
                     remote_response.status,
                     remote_response.data.decode('utf-8'),
                     remote_response,)
    return HttpResponse(content=remote_response.data, status=remote_response.status)


def oauth_callback(request, **kwargs):
    logger.info("A Login callback received")
    logger.debug("data %s", request.GET)
    # Prepare request data
    url = settings.OAUTH_SERVER_URL + "o/token/"
    code = request.GET["code"]
    call_back_url = "http://" + request.META['HTTP_HOST']+reverse('django_navbar_client:callback')
    fields = {
        "grant_type": "authorization_code",
        "client_id": settings.OAUTH_CLIENT_ID,
        "client_secret": settings.OAUTH_CLIENT_SECRET,
        "code": code,
        "redirect_uri": call_back_url,
    }
    logger.info("Asking Auth server for the Token")

    r = PM.request(method='post', url=url, fields=fields)
    if r.status == 200:
        logger.debug("\tURL: %s", url)
        logger.debug("\tFIELDS: %s", fields)
        logger.debug("Response Obtained \n\t Data: %s", r.data)
        data = loads(r.data.decode('utf-8'))
        access_token = data["token_type"] + " " + data["access_token"]
        logger.info("Asking Auth server for ME")
        url = settings.OAUTH_SERVER_URL + "api/me"
        r = ask_oauth(url, token=access_token)
        if r.status == 200:
            logger.debug("Response Obtained \n\t Data: %s", r.data)
            me = loads(r.data.decode('utf-8'))
            try:
                user = User.objects.get_by_natural_key(me["user"])
            except ObjectDoesNotExist:
                logger.info("Local user created for %s", me["user"])
                user = User.objects.create_user(username=me["user"])
            try:
                user.authprofile.token = access_token
                logger.debug("Set profile token=%s", access_token)
                user.authprofile.uuid = me["uuid"]
                logger.debug("Set profile UUID=%s", me["uuid"])
            except AttributeError:
                logger.info("Local user created for %s", me["user"])
                AuthProfile.objects.create(user=user, token=access_token, uuid=me["uuid"])
            logger.debug("Saving user and profile")
            user.save()
            user.authprofile.save()
            logger.debug("Login user into system")
            login(request, user, backend='django.contrib.auth.backends.ModelBackend')
            logger.debug("Logged as s% in session %s ", request.user.id, request.session.session_key)
            return redirect(kwargs.get("next", reverse('home')))
        else:
            logger.warning("Something went wrong asking ME: %i", r.status)
            logger.warning("\tURL: %s", url)
            logger.debug(r.data)
    else:
        logger.warning("Something went wrong asking token: %i", r.status)
        logger.warning("\tURL: %s", url)
        logger.warning("\tFIELDS: %s", fields)
        logger.debug(r.data)
    logger.info("Unsuccessful login")
    response = HttpResponse(content=r.data, status=r.status)
    return response


def ask_oauth(url, token, refesh=None):
    headers = None
    if token:
        headers = {"Authorization": token}
        logger.debug("\tHEADERS   : %s", headers)
    logger.debug("\tURL         : %s", url)
    response = PM.request(method='get', url=url, headers=headers)
    return response



from django.urls import path
from django_navbar_client.views import oauth_callback, oauth_login, oauth_logout, oauth_navbar


app_name = "django_navbar_client"

urlpatterns = [
    path('api/login_callback/', oauth_callback, name="callback"),
    path('view/logout/', oauth_logout, name="logout"),
    path('view/login/', oauth_login, name="logout"),
    path('widget/navbar/', oauth_navbar, name="navbar"),
]



# Generated by Django 2.1.1 on 2018-09-04 15:14

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0009_alter_user_last_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='AuthProfile',
            fields=[
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, primary_key=True, serialize=False, to=settings.AUTH_USER_MODEL)),
                ('uuid', models.UUIDField(blank=True, default=uuid.UUID('00000000-0000-0000-0000-000000000000'), editable=False)),
                ('token', models.TextField()),
            ],
        ),
    ]



from django.conf import settings
from django.contrib.auth.models import User
from django.db import models
from uuid import uuid5,UUID

# Create your models here.
NIL = UUID("00000000000000000000000000000000")


class AuthProfile(models.Model):
    user = models.OneToOneField(User, models.CASCADE, primary_key=True)
    uuid = models.UUIDField(blank=True, default=NIL, editable=False)
    token = models.TextField()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def save(self, *args, **kwargs):
        if self.uuid == NIL:
            self.uuid = uuid5(settings.NAMESPACE_HOST_UUID, self.user.username)
        super().save(*args, **kwargs)

    def __str__(self):
        return "{0}::{1}".format(self.user.username, self.uuid)



"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='django_navbar_client',  
    version='0.9.10',
    description='Easy remote oauth identification', 
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),  
    install_requires=['django'],

    keywords='Fiware Orion development', 
    long_description=long_description, 
    long_description_content_type='text/markdown',  
    url='https://github.com/deusto-tech/django-oauth-client/', 
    author='Josu Bermudez', 
    author_email='josu.bermudez@deusto.es',  
    project_urls={ 
        'Bug Reports': 'https://github.com/deusto-tech/django-oauth-client//issues',
        'Source': 'https://github.com/deusto-tech/django-oauth-client',
    },

    classifiers=[  
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',

        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Framework :: Django :: 2.0',
    ],
    )



from django.contrib.auth import login
from django.contrib.auth.models import User
from logging import getLogger
from django_navbar_client.models import AuthProfile

logger = getLogger(__name__)


class CreateAnonymousUser:
    def __init__(self, get_response):
        self.get_response = get_response
        # One-time configuration and initialization.

    def __call__(self, request):
        # Code to be executed for each request before
        # the view (and later middleware) are called.
        if request.user.is_anonymous:
            user, found = User.objects.get_or_create(username="anonymous")
            request.user = user

        response = self.get_response(request)

        # Code to be executed for each request/response after
        # the view is called.

        return response


class ParamUUIDUserMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        uuid = request.GET.get("UUID", request.GET.get("uuid", False))
        try:
            if uuid and (request.user.is_anonymous or (
                    request.user.authprofile and request.user.authprofile.uuid != uuid)):
                user = AuthProfile.objects.get(uuid=uuid).user
                if request.user.is_authenticated:
                    logger.info("User override {} Now is {}", request.user.username, user)
                login(request, user, backend='oauth2_provider.backends.OAuth2Backend')
                request.user = request._cached_user = user
        except (AuthProfile.DoesNotExist, ):
            logger.debug("User {} not found", uuid)
        return self.get_response(request)
